@@ -183,15 +183,22 @@ int mutt_display_message (HEADER *cur)
 	else
 	  mutt_error ( _("S/MIME certificate owner does not match sender."));
       }
+      else if (cur->security & PARTSIGN)
+	mutt_message (_("Warning: Part of this message has not been signed."));
       else if (cur->security & SIGN || cur->security & BADSIGN)
 	mutt_error ( _("S/MIME signature could NOT be verified."));
     }
 
     if (WithCrypto 
         && (cur->security & APPLICATION_PGP) && (cmflags & M_CM_VERIFY))
-      mutt_message ((cur->security & GOODSIGN) ?
-		    _("PGP signature successfully verified.") :
-		    _("PGP signature could NOT be verified."));
+    {
+      if (cur->security & GOODSIGN)
+	mutt_message (_("PGP signature successfully verified."));
+      else if (cur->security & PARTSIGN)
+	mutt_message (_("Warning: Part of this message has not been signed."));
+      else
+	mutt_message (_("PGP signature could NOT be verified."));
+    }
 
     /* Invoke the builtin pager */
     memset (&info, 0, sizeof (pager_t));
@@ -275,10 +282,10 @@ void ci_bounce_message (HEADER *h, int *redraw)
     mutt_format_string (prompt, sizeof (prompt),
 			0, COLS-extra_space, 0, 0,
 			prompt, sizeof (prompt), 0);
-    strncat (prompt, "...?", sizeof (prompt));
+    safe_strcat (prompt, sizeof (prompt), "...?");
   }
   else
-    strncat (prompt, "?", sizeof (prompt));
+    safe_strcat (prompt, sizeof (prompt), "?");
 
   if (query_quadoption (OPT_BOUNCE, prompt) != M_YES)
   {
@@ -316,7 +323,7 @@ static void pipe_set_flags (int decode, int print, int *cmflags, int *chflags)
   
 }
 
-void pipe_msg (HEADER *h, FILE *fp, int decode, int print)
+static void pipe_msg (HEADER *h, FILE *fp, int decode, int print)
 {
   int cmflags = 0;
   int chflags = CH_FROM;
@@ -494,9 +501,9 @@ int mutt_select_sort (int reverse)
   int method = Sort; /* save the current method in case of abort */
 
   switch (mutt_multi_choice (reverse ?
-			     _("Rev-Sort (d)ate/(f)rm/(r)ecv/(s)ubj/t(o)/(t)hread/(u)nsort/si(z)e/s(c)ore?: ") :
-			     _("Sort (d)ate/(f)rm/(r)ecv/(s)ubj/t(o)/(t)hread/(u)nsort/si(z)e/s(c)ore?: "),
-			     _("dfrsotuzc")))
+			     _("Rev-Sort (d)ate/(f)rm/(r)ecv/(s)ubj/t(o)/(t)hread/(u)nsort/si(z)e/s(c)ore/s(p)am?: ") :
+			     _("Sort (d)ate/(f)rm/(r)ecv/(s)ubj/t(o)/(t)hread/(u)nsort/si(z)e/s(c)ore/s(p)am?: "),
+			     _("dfrsotuzcp")))
   {
   case -1: /* abort - don't resort */
     return -1;
@@ -536,6 +543,10 @@ int mutt_select_sort (int reverse)
   case 9: /* s(c)ore */ 
     Sort = SORT_SCORE;
     break;
+
+  case 10: /* s(p)am */
+    Sort = SORT_SPAM;
+    break;
   }
   if (reverse)
     Sort |= SORT_REVERSE;
@@ -641,16 +652,17 @@ static void set_copy_flags (HEADER *hdr, int decode, int decrypt, int *cmflags,
   {
     *chflags = CH_XMIT | CH_MIME | CH_TXTPLAIN;
     *cmflags = M_CM_DECODE | M_CM_CHARCONV;
-  }
 
-  /* respect $weed only if decode doesn't kick in
-   * for decrypt.
-   */
+    if (!decrypt)	/* If decode doesn't kick in for decrypt, */
+    {
+      *chflags |= CH_DECODE;	/* then decode RFC 2047 headers, */
 
-  if (decode && !decrypt && option (OPTWEED))
-  {
-    *chflags |= CH_WEED;
-    *cmflags |= M_CM_WEED;
+      if (option (OPTWEED))
+      {
+	*chflags |= CH_WEED;	/* and respect $weed. */
+	*cmflags |= M_CM_WEED;
+      }
+    }
   }
 }
 
@@ -898,12 +910,17 @@ void mutt_edit_content_type (HEADER *h, BODY *b, FILE *fp)
 
   /* inform the user */
   
+  snprintf (tmp, sizeof (tmp), "%s/%s", TYPE (b), NONULL (b->subtype));
   if (type_changed)
     mutt_message (_("Content-Type changed to %s."), tmp);
-  else if (b->type == TYPETEXT && charset_changed)
+  if (b->type == TYPETEXT && charset_changed)
+  {
+    if (type_changed)
+      mutt_sleep (1);
     mutt_message (_("Character set changed to %s; %s."),
 		  mutt_get_parameter ("charset", b->parameter),
 		  b->noconv ? _("not converting") : _("converting"));
+  }
 
   b->force_charset |= charset_changed ? 1 : 0;
 
@@ -933,6 +950,8 @@ static int _mutt_check_traditional_pgp (HEADER *h, int *redraw)
   MESSAGE *msg;
   int rv = 0;
   
+  h->security |= PGP_TRADITIONAL_CHECKED;
+  
   mutt_parse_mime_message (Context, h);
   if ((msg = mx_open_message (Context, h->msgno)) == NULL)
     return 0;
@@ -951,12 +970,13 @@ int mutt_check_traditional_pgp (HEADER *h, int *redraw)
 {
   int i;
   int rv = 0;
-  if (h)
+  if (h && !(h->security & PGP_TRADITIONAL_CHECKED))
     rv = _mutt_check_traditional_pgp (h, redraw);
   else
   {
     for (i = 0; i < Context->vcount; i++)
-      if (Context->hdrs[Context->v2r[i]]->tagged)
+      if (Context->hdrs[Context->v2r[i]]->tagged && 
+	  !(Context->hdrs[Context->v2r[i]]->security & PGP_TRADITIONAL_CHECKED))
 	rv = _mutt_check_traditional_pgp (Context->hdrs[Context->v2r[i]], redraw)
 	  || rv;
   }
