@@ -4,6 +4,7 @@
  * Copyright (C) 2001  Thomas Roessler <roessler@does-not-exist.org>
  *                     Oliver Ehli <elmy@acm.org>
  * Copyright (C) 2003  Werner Koch <wk@gnupg.org>
+ * Copyright (C) 2004 g10code GmbH
  *
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -107,64 +108,28 @@ static void disable_coredumps (void)
 int crypt_valid_passphrase(int flags)
 {
   time_t now = time (NULL);
+  int ret = 0;
 
 # if defined(HAVE_SETRLIMIT) &&(!defined(DEBUG))
   disable_coredumps ();
 # endif
 
   if ((WithCrypto & APPLICATION_PGP) && (flags & APPLICATION_PGP))
-  {
-    extern char PgpPass[STRING];
-    extern time_t PgpExptime;
-
-    if (pgp_use_gpg_agent())
-    {
-      *PgpPass = 0;
-      return 1; /* handled by gpg-agent */
-    }
-
-    if (now < PgpExptime) return 1; /* just use the cached copy. */
-    crypt_pgp_void_passphrase ();
-      
-    if (mutt_get_password (_("Enter PGP passphrase:"),
-                           PgpPass, sizeof (PgpPass)) == 0)
-    {
-      PgpExptime = time (NULL) + PgpTimeout;
-      return (1);
-    }
-    else
-      PgpExptime = 0;
-    }
+    ret = crypt_pgp_valid_passphrase ();
 
   if ((WithCrypto & APPLICATION_SMIME) && (flags & APPLICATION_SMIME))
-  {
-    extern char SmimePass[STRING];
-    extern time_t SmimeExptime;
+    ret = crypt_smime_valid_passphrase ();
 
-    if (now < SmimeExptime) return (1);
-    crypt_smime_void_passphrase ();
-      
-    if (mutt_get_password (_("Enter SMIME passphrase:"), SmimePass,
-			   sizeof (SmimePass)) == 0)
-    {
-      SmimeExptime = time (NULL) + SmimeTimeout;
-      return (1);
-    }
-    else
-      SmimeExptime = 0;
-  }
-
-  return (0);
+  return ret;
 }
 
 
 
-int mutt_protect (HEADER *msg, HEADER *cur, char *keylist)
+int mutt_protect (HEADER *msg, char *keylist)
 {
   BODY *pbody = NULL, *tmp_pbody = NULL;
   BODY *tmp_smime_pbody = NULL;
   BODY *tmp_pgp_pbody = NULL;
-  int traditional = 0;
   int flags = (WithCrypto & APPLICATION_PGP)? msg->security: 0;
   int i;
 
@@ -174,42 +139,33 @@ int mutt_protect (HEADER *msg, HEADER *cur, char *keylist)
   if ((msg->security & SIGN) && !crypt_valid_passphrase (msg->security))
     return (-1);
 
-  if ((WithCrypto & APPLICATION_PGP) && (msg->security & APPLICATION_PGP))
+  if ((WithCrypto & APPLICATION_PGP) && ((msg->security & PGPINLINE) == PGPINLINE))
   {
-    if ((msg->content->type == TYPETEXT) &&
-	!ascii_strcasecmp (msg->content->subtype, "plain"))
+    /* they really want to send it inline... go for it */
+    if (!isendwin ()) mutt_endwin _("Invoking PGP...");
+    pbody = crypt_pgp_traditional_encryptsign (msg->content, flags, keylist);
+    if (pbody)
     {
-      if (cur && cur->security && option (OPTPGPAUTOTRAD)
-	  && (option (OPTCRYPTREPLYENCRYPT)
-	      || option (OPTCRYPTREPLYSIGN)
-	      || option (OPTCRYPTREPLYSIGNENCRYPTED)))
-	{
-	  if(mutt_is_application_pgp(cur->content))
-	    traditional = 1;
-	}
-      else
-	{
-	  if ((i = query_quadoption (OPT_PGPTRADITIONAL, _("Create a traditional (inline) PGP message?"))) == -1)
-	    return -1;
-	  else if (i == M_YES)
-	    traditional = 1;
-	}
-    }
-    if (traditional)
-    {
-      if (!isendwin ()) mutt_endwin _("Invoking PGP...");
-      if (!(pbody = crypt_pgp_traditional_encryptsign (msg->content, flags, keylist)))
-	return -1;
-
       msg->content = pbody;
       return 0;
     }
+
+    /* otherwise inline won't work...ask for revert */
+    if ((i = query_quadoption (OPT_PGPMIMEAUTO, _("Message can't be sent inline.  Revert to using PGP/MIME?"))) != M_YES)
+      {
+	mutt_error _("Mail not sent.");
+	return -1;
+      }
+
+    /* go ahead with PGP/MIME */
   }
 
   if (!isendwin ()) mutt_endwin (NULL);
 
   if ((WithCrypto & APPLICATION_SMIME))
     tmp_smime_pbody = msg->content;
+  if ((WithCrypto & APPLICATION_PGP))
+    tmp_pgp_pbody   = msg->content;
 
   if (msg->security & SIGN)
   {
@@ -267,7 +223,7 @@ int mutt_protect (HEADER *msg, HEADER *cur, char *keylist)
     if ((WithCrypto & APPLICATION_PGP)
         && (msg->security & APPLICATION_PGP))
     {
-      if (!(pbody = crypt_pgp_encrypt_message (msg->content, keylist,
+      if (!(pbody = crypt_pgp_encrypt_message (tmp_pgp_pbody, keylist,
                                                flags & SIGN)))
       {
 
@@ -275,9 +231,9 @@ int mutt_protect (HEADER *msg, HEADER *cur, char *keylist)
 	if (flags != msg->security)
 	{
 	  /* remove the outer multipart layer */
-	  msg->content = mutt_remove_multipart (msg->content);
+	  tmp_pgp_pbody = mutt_remove_multipart (tmp_pgp_pbody);
 	  /* get rid of the signature */
-	  mutt_free_body (&msg->content->next);
+	  mutt_free_body (&tmp_pgp_pbody->next);
 	}
 
 	return (-1);
@@ -285,11 +241,12 @@ int mutt_protect (HEADER *msg, HEADER *cur, char *keylist)
 
       /* destroy temporary signature envelope when doing retainable 
        * signatures.
+
        */
       if (flags != msg->security)
       {
-	mutt_remove_multipart (msg->content);
-	mutt_free_body (&msg->content->next);
+	tmp_pgp_pbody = mutt_remove_multipart (tmp_pgp_pbody);
+	mutt_free_body (&tmp_pgp_pbody->next);
       }
     }
   }
@@ -389,6 +346,9 @@ int mutt_is_application_pgp (BODY *m)
     else if (p && !ascii_strncasecmp ("pgp-keys", p, 7))
       t |= PGPKEY;
   }
+  if (t)
+    t |= PGPINLINE;
+
   return t;
 }
 
@@ -466,6 +426,9 @@ int crypt_query (BODY *m)
 
   if (!WithCrypto)
     return 0;
+  
+  if (!m)
+    return 0;
 
   if (m->type == TYPEAPPLICATION)
   {
@@ -491,15 +454,27 @@ int crypt_query (BODY *m)
     t |= mutt_is_multipart_encrypted(m);
     t |= mutt_is_multipart_signed (m);
 
-    if (t && m->goodsig) t |= GOODSIGN;
+    if (t && m->goodsig) 
+      t |= GOODSIGN;
   }
 
   if (m->type == TYPEMULTIPART || m->type == TYPEMESSAGE)
   {
     BODY *p;
+    int u, v, w;
+    
+    u = m->parts ? 0xffffffff : 0;	/* Bits set in all parts */
+    w = 0;				/* Bits set in any part  */
  
     for (p = m->parts; p; p = p->next)
-      t |= crypt_query (p) & ~GOODSIGN;
+    {
+      v  = crypt_query (p);
+      u &= v; w |= v;
+    }
+    t |= u | (w & ~GOODSIGN);
+    
+    if ((w & GOODSIGN) && !(u & GOODSIGN))
+      t |= PARTSIGN;
   }
 
   return t;
@@ -871,7 +846,8 @@ void mutt_signed_handler (BODY *a, STATE *s)
 
 	  if ((WithCrypto & APPLICATION_SMIME)
               && signatures[i]->type == TYPEAPPLICATION 
-	      && !mutt_strcasecmp(signatures[i]->subtype, "x-pkcs7-signature"))
+	      && (!mutt_strcasecmp(signatures[i]->subtype, "x-pkcs7-signature")
+		  || !mutt_strcasecmp(signatures[i]->subtype, "pkcs7-signature")))
 	  {
 	    if (crypt_smime_verify_one (signatures[i], s, tempfile) != 0)
 	      goodsig = 0;
@@ -888,7 +864,7 @@ void mutt_signed_handler (BODY *a, STATE *s)
       mutt_unlink (tempfile);
 
       b->goodsig = goodsig;
-      b->badsig = goodsig;		/* XXX - WHAT!?!?!? */
+      b->badsig  = !goodsig;
       
       /* Now display the signed body */
       state_attach_puts (_("[-- The following data is signed --]\n\n"), s);
