@@ -17,10 +17,8 @@
  */
 
 #include "mutt.h"
-
-#ifdef HAVE_PGP
-#include "pgp.h"
-#endif
+#include "mutt_crypt.h"
+#include "mutt_idna.h"
 
 #include <sys/stat.h>
 #include <string.h>
@@ -48,7 +46,8 @@ void mutt_edit_headers (const char *editor,
     mutt_perror (path);
     return;
   }
-
+  
+  mutt_env_to_local (msg->env);
   mutt_write_rfc822_header (ofp, msg->env, NULL, 1, 0);
   fputc ('\n', ofp);	/* tie off the header. */
 
@@ -178,15 +177,14 @@ void mutt_edit_headers (const char *editor,
     }
 
 
-
-#ifdef HAVE_PGP
-    else if (ascii_strncasecmp ("pgp:", cur->data, 4) == 0)
+    else if ((WithCrypto & APPLICATION_PGP)
+             &&ascii_strncasecmp ("pgp:", cur->data, 4) == 0)
     {
       msg->security = mutt_parse_crypt_hdr (cur->data + 4, 0);
+      if (msg->security)
+	msg->security |= APPLICATION_PGP;
       keep = 0;
     }
-#endif
-
 
     if (keep)
     {
