@@ -31,7 +31,7 @@ static void hmac_md5 (const char* password, char* challenge,
   unsigned char* response);
 
 /* imap_auth_cram_md5: AUTH=CRAM-MD5 support. */
-imap_auth_res_t imap_auth_cram_md5 (IMAP_DATA* idata)
+imap_auth_res_t imap_auth_cram_md5 (IMAP_DATA* idata, const char* method)
 {
   char ibuf[LONG_STRING*2], obuf[LONG_STRING];
   unsigned char hmac_response[MD5_DIGEST_LEN];
@@ -104,7 +104,7 @@ imap_auth_res_t imap_auth_cram_md5 (IMAP_DATA* idata)
   
   mutt_to_base64 ((unsigned char*) ibuf, (unsigned char*) obuf, strlen (obuf),
 		  sizeof (ibuf) - 2);
-  strncat (ibuf, "\r\n", sizeof (ibuf));
+  safe_strcat (ibuf, sizeof (ibuf), "\r\n");
   mutt_socket_write (idata->conn, ibuf);
 
   do
@@ -122,7 +122,7 @@ imap_auth_res_t imap_auth_cram_md5 (IMAP_DATA* idata)
 
  bail:
   mutt_error _("CRAM-MD5 authentication failed.");
-  sleep (2);
+  mutt_sleep (2);
   return IMAP_AUTH_FAILURE;
 }
 
