@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2 Brendan Cully <brendan@kublai.com>
+ * Copyright (C) 2000-3 Brendan Cully <brendan@kublai.com>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -37,7 +37,7 @@ imap_auth_res_t imap_auth_sasl (IMAP_DATA* idata, const char* method)
   sasl_conn_t* saslconn;
   sasl_interact_t* interaction = NULL;
   int rc, irc;
-  char buf[LONG_STRING];
+  char buf[HUGE_STRING];
   const char* mech;
 #ifdef USE_SASL2
   const char *pc = NULL;
@@ -109,19 +109,26 @@ imap_auth_res_t imap_auth_sasl (IMAP_DATA* idata, const char* method)
     return IMAP_AUTH_UNAVAIL;
   }
 
-  mutt_message _("Authenticating (SASL)...");
+  mutt_message (_("Authenticating (%s)..."), mech);
 
   snprintf (buf, sizeof (buf), "AUTHENTICATE %s", mech);
   imap_cmd_start (idata, buf);
   irc = IMAP_CMD_CONTINUE;
 
   /* looping protocol */
-  while (rc == SASL_CONTINUE)
+  while (rc == SASL_CONTINUE || olen > 0)
   {
     do
       irc = imap_cmd_step (idata);
     while (irc == IMAP_CMD_CONTINUE);
 
+    if (method && irc == IMAP_CMD_NO)
+    {
+      dprint (2, (debugfile, "imap_auth_sasl: %s failed\n", method));
+      sasl_dispose (&saslconn);
+      return IMAP_AUTH_UNAVAIL;
+    }
+
     if (irc == IMAP_CMD_BAD || irc == IMAP_CMD_NO)
       goto bail;
 
@@ -153,7 +160,7 @@ imap_auth_res_t imap_auth_sasl (IMAP_DATA* idata, const char* method)
       client_start = 0;
 
     /* send out response, or line break if none needed */
-    if (pc)
+    if (olen)
     {
       if (sasl_encode64 (pc, olen, buf, sizeof (buf), &olen) != SASL_OK)
       {
@@ -168,7 +175,7 @@ imap_auth_res_t imap_auth_sasl (IMAP_DATA* idata, const char* method)
 #endif
     }
     
-    if (olen || rc == SASL_CONTINUE)
+    if (irc == IMAP_CMD_RESPOND)
     {
       strfcpy (buf + olen, "\r\n", sizeof (buf) - olen);
       mutt_socket_write (idata->conn, buf);
@@ -180,6 +187,8 @@ imap_auth_res_t imap_auth_sasl (IMAP_DATA* idata, const char* method)
       mutt_socket_write (idata->conn, "*\r\n");
       dprint (1, (debugfile, "imap_auth_sasl: sasl_client_step error %d\n",rc));
     }
+	  
+    olen = 0;
   }
 
   while (irc != IMAP_CMD_OK)
