@@ -54,6 +54,14 @@ void *safe_calloc (size_t nmemb, size_t size)
 
   if (!nmemb || !size)
     return NULL;
+
+  if (((size_t) -1) / nmemb <= size)
+  {
+    mutt_error _("Integer overflow -- can't allocate memory!");
+    sleep (1);
+    mutt_exit (1);
+  }
+  
   if (!(p = calloc (nmemb, size)))
   {
     mutt_error _("Out of memory!");
@@ -78,9 +86,10 @@ void *safe_malloc (size_t siz)
   return (p);
 }
 
-void safe_realloc (void **p, size_t siz)
+void safe_realloc (void *ptr, size_t siz)
 {
   void *r;
+  void **p = (void **)ptr;
 
   if (siz == 0)
   {
@@ -110,8 +119,9 @@ void safe_realloc (void **p, size_t siz)
   *p = r;
 }
 
-void safe_free (void **p)
+void safe_free (void *ptr)
 {
+  void **p = (void **)ptr;
   if (*p)
   {
     free (*p);				/* __MEM_CHECKED__ */
@@ -143,6 +153,45 @@ char *safe_strdup (const char *s)
   return (p);
 }
 
+char *safe_strcat (char *d, size_t l, const char *s)
+{
+  char *p = d;
+
+  if (!l) 
+    return d;
+
+  l--; /* Space for the trailing '\0'. */
+  
+  for (; *d && l; l--)
+    d++;
+  for (; *s && l; l--)
+    *d++ = *s++;
+
+  *d = '\0';
+  
+  return p;
+}
+
+char *safe_strncat (char *d, size_t l, const char *s, size_t sl)
+{
+  char *p = d;
+
+  if (!l)
+    return d;
+  
+  l--; /* Space for the trailing '\0'. */
+  
+  for (; *d && l; l--)
+    d++;
+  for (; *s && l && sl; l--, sl--)
+    *d++ = *s++;
+
+  *d = '\0';
+  
+  return p;
+}
+
+
 void mutt_str_replace (char **p, const char *s)
 {
   FREE (p);
@@ -152,7 +201,7 @@ void mutt_str_replace (char **p, const char *s)
 void mutt_str_adjust (char **p)
 {
   if (!p || !*p) return;
-  safe_realloc ((void **) p, strlen (*p) + 1);
+  safe_realloc (p, strlen (*p) + 1);
 }
 
 /* convert all characters in the string to lowercase */
@@ -162,7 +211,7 @@ char *mutt_strlower (char *s)
 
   while (*p)
   {
-    *p = tolower (*p);
+    *p = tolower ((unsigned char) *p);
     p++;
   }
 
@@ -171,13 +220,33 @@ char *mutt_strlower (char *s)
 
 void mutt_unlink (const char *s)
 {
+  int fd;
+  int flags;
   FILE *f;
-  struct stat sb;
+  struct stat sb, sb2;
   char buf[2048];
+
+  /* Defend against symlink attacks */
   
-  if (stat (s, &sb) == 0)
+#ifdef O_NOFOLLOW 
+  flags = O_RDWR | O_NOFOLLOW;
+#else
+  flags = O_RDWR;
+#endif
+  
+  if (lstat (s, &sb) == 0 && S_ISREG(sb.st_mode))
   {
-    if ((f = fopen (s, "r+")))
+    if ((fd = open (s, flags)) < 0)
+      return;
+    
+    if ((fstat (fd, &sb2) != 0) || !S_ISREG (sb2.st_mode) 
+	|| (sb.st_dev != sb2.st_dev) || (sb.st_ino != sb2.st_ino))
+    {
+      close (fd);
+      return;
+    }
+    
+    if ((f = fdopen (fd, "r+")))
     {
       unlink (s);
       memset (buf, 0, sizeof (buf));
@@ -484,7 +553,7 @@ char *mutt_read_line (char *s, size_t *size, FILE *fp, int *line)
         /* There wasn't room for the line -- increase ``s'' */
         offset = *size - 1; /* overwrite the terminating 0 */
         *size += STRING;
-        safe_realloc ((void **) &s, *size);
+        safe_realloc (&s, *size);
       }
     }
   }
@@ -600,7 +669,10 @@ const char *mutt_stristr (const char *haystack, const char *needle)
 
   while (*(p = haystack))
   {
-    for (q = needle; *p && *q && tolower (*p) == tolower (*q); p++, q++)
+    for (q = needle;
+         *p && *q &&
+           tolower ((unsigned char) *p) == tolower ((unsigned char) *q);
+         p++, q++)
       ;
     if (!*q)
       return (haystack);
@@ -633,3 +705,12 @@ char *mutt_concat_path (char *d, const char *dir, const char *fname, size_t l)
   snprintf (d, l, fmt, dir, fname);
   return d;
 }
+
+const char *mutt_basename (const char *f)
+{
+  const char *p = strrchr (f, '/');
+  if (p)
+    return p + 1;
+  else
+    return f;
+}
