@@ -1,6 +1,7 @@
 /*
  * Copyright (C) 1996-2002 Michael R. Elkins <me@mutt.org>
  * Copyright (C) 1999-2002 Thomas Roessler <roessler@does-not-exist.org>
+ * Copyright (C) 2004 g10 Code GmbH
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -210,9 +211,15 @@ static void show_version (void)
 #endif
 
 #ifdef USE_FLOCK
-	"+USE_FLOCK"
+	"+USE_FLOCK   "
 #else
-	"-USE_FLOCK"
+	"-USE_FLOCK   "
+#endif
+	
+#ifdef USE_INODESORT
+	"+USE_INODESORT   "
+#else
+	"-USE_INODESORT   "
 #endif
 	);
   puts (
@@ -241,6 +248,12 @@ static void show_version (void)
 	"-USE_SSL  "
 #endif
 
+#ifdef USE_GNUTLS
+	"+USE_GNUTLS  "
+#else
+	"-USE_GNUTLS  "
+#endif
+
 #ifdef USE_SASL
 	"+USE_SASL  "
 #else
@@ -411,6 +424,12 @@ static void show_version (void)
 	"-HAVE_GETADDRINFO  "
 #endif
 
+#if USE_HCACHE
+	"+USE_HCACHE  "
+#else
+	"-USE_HCACHE  "
+#endif
+
 	);
 
 #ifdef ISPELL
@@ -444,6 +463,7 @@ static void start_curses (void)
 #ifdef USE_SLANG_CURSES
   SLtt_Ignore_Beep = 1; /* don't do that #*$@^! annoying visual beep! */
   SLsmg_Display_Eight_Bit = 128; /* characters above this are printable */
+  SLtt_set_color(0, NULL, "default", "default");
 #else
   /* should come before initscr() so that ncurses 4.2 doesn't try to install
      its own SIGWINCH handler */
@@ -653,6 +673,9 @@ int main (int argc, char **argv)
   mutt_init (flags & M_NOSYSRC, commands);
   mutt_free_list (&commands);
 
+  /* Initialize crypto backends.  */
+  crypt_init ();
+
   if (queries)
     return mutt_query_variables (queries);
 
