@@ -22,13 +22,11 @@
 #include "globals.h"
 #include "mutt_socket.h"
 #include "mutt_tunnel.h"
-#ifdef USE_SSL
+#if defined(USE_SSL) || defined(USE_GNUTLS) || defined(USE_NSS)
 # include "mutt_ssl.h"
 #endif
 
-#ifdef HAVE_LIBIDN
-#include <idna.h>
-#endif
+#include "mutt_idna.h"
 
 #include <unistd.h>
 #include <netinet/in.h>
@@ -259,6 +257,12 @@ CONNECTION* mutt_conn_find (const CONNECTION* start, const ACCOUNT* account)
     ssl_socket_setup (conn);
 #elif USE_NSS
     mutt_nss_socket_setup (conn);
+#elif USE_GNUTLS
+    if (mutt_gnutls_socket_setup (conn) < 0)
+    {
+      mutt_socket_free (conn);
+      return NULL;
+    }
 #else
     mutt_error _("SSL is unavailable.");
     mutt_sleep (2);
@@ -413,7 +417,7 @@ int raw_socket_open (CONNECTION* conn)
   snprintf (port, sizeof (port), "%d", conn->account.port);
   
 # ifdef HAVE_LIBIDN
-  if (idna_to_ascii_from_locale (conn->account.host, &host_idna, 1, 0) != IDNA_SUCCESS)
+  if (idna_to_ascii_lz (conn->account.host, &host_idna, 1) != IDNA_SUCCESS)
   {
     mutt_error (_("Bad IDN \"%s\"."), conn->account.host);
     return -1;
@@ -470,7 +474,7 @@ int raw_socket_open (CONNECTION* conn)
   sin.sin_family = AF_INET;
 
 # ifdef HAVE_LIBIDN
-  if (idna_to_ascii_from_locale (conn->account.host, &host_idna, 1, 0) != IDNA_SUCCESS)
+  if (idna_to_ascii_lz (conn->account.host, &host_idna, 1) != IDNA_SUCCESS)
   {
     mutt_error (_("Bad IDN \"%s\"."), conn->account.host);
     return -1;
@@ -483,13 +487,17 @@ int raw_socket_open (CONNECTION* conn)
 
   if ((he = gethostbyname (host_idna)) == NULL)
   {
+# ifdef HAVE_LIBIDN
     FREE (&host_idna);
+# endif
     mutt_error (_("Could not find the host \"%s\""), conn->account.host);
 	
     return -1;
   }
-  
+
+# ifdef HAVE_LIBIDN
   FREE (&host_idna);
+# endif
 
   mutt_message (_("Connecting to %s..."), conn->account.host); 
 
